﻿using Infragistics.Controls.Schedules;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Threading;

namespace TestScheduleView
{
    public class MainWindowViewModel: INotifyPropertyChanged
    {
        private Dispatcher _dispatcher;
        private BackgroundWorker _worker;
        private ObservableCollection<Appointment> _appointments;
        private ObservableCollection<ResourceCalendar> _calendars;
        private ObservableCollection<Resource> _resources;
        private double _memory;
        private int _dataGenerationRate;

        public event PropertyChangedEventHandler PropertyChanged;

        public MainWindowViewModel()
        {
            _dataGenerationRate = 2;
            _dispatcher = Application.Current.Dispatcher;
            _appointments = new ObservableCollection<Appointment>();
            _calendars = new ObservableCollection<ResourceCalendar>();
            _resources = new ObservableCollection<Resource>();

            
            GenerateData();

            _worker = new BackgroundWorker();
            _worker.DoWork += worker_DoWork;
            _worker.RunWorkerAsync();
        }

        private Dispatcher Dispatcher
        {
            get { return _dispatcher; }
        }

        public ObservableCollection<Appointment> Appointments
        {
            get { return _appointments; }
        }

        public ObservableCollection<ResourceCalendar> Calendars
        {
            get { return _calendars; }
        }

        public ObservableCollection<Resource> Resources
        {
            get { return _resources; }
        }

        public TimeSpan SlotInterval
        {
            get { return TimeSpan.FromDays(1); }
        }

        public double Memory
        {
            get { return _memory; }
            set
            {
                _memory=value;
                OnPropertyChanged("Memory");
            }
        }

        public string DataGenerationRateDisplay
        {
            get
            {
                switch (_dataGenerationRate)
                {
                    case 1: return "Paused";
                    case 2: return "Normal";
                    case 3: return "Fast";
                    default: return "?";
                }
            }
        }

        public int DataGenerationRate
        {
            get { return _dataGenerationRate; }
            set
            {
                _dataGenerationRate = value;
                OnPropertyChanged("DataGenerationRate");
                OnPropertyChanged("DataGenerationRateDisplay");
            }
        }

        void worker_DoWork(object sender, DoWorkEventArgs e)
        {
            // Generate random data
            while (true)
            {
                int timeout = _dataGenerationRate == 2 ? 1000 : 100;
                Thread.Sleep(timeout);

                if (_dataGenerationRate != 1)
                    GenerateData();
            }
        }

        private void GenerateData()
        {
            Dispatch(() => GenerateResources());
            Dispatch(() => GenerateResourceCalendars());
            GenerateAppointments();
            SetMemoryUsage();
        }

        private void GenerateResources()
        {
            _resources.Clear();

            _resources.Add(new Resource() { Id = "Resource0", Name = "A", IsLocked = true });
            _resources.Add(new Resource() { Id = "Resource1", Name = "B", IsLocked = true });
            _resources.Add(new Resource() { Id = "Resource2", Name = "C", IsLocked = true });
            _resources.Add(new Resource() { Id = "Resource3", Name = "D", IsLocked = true });
            _resources.Add(new Resource() { Id = "Resource4", Name = "E", IsLocked = true });
        }

        private void GenerateResourceCalendars()
        {
            _calendars.Clear();

            _calendars.Add(new ResourceCalendar() { Id = "Calendar0", OwningResourceId = "Resource0", IsVisible = true });
            _calendars.Add(new ResourceCalendar() { Id = "Calendar1", OwningResourceId = "Resource1", IsVisible = true });
            _calendars.Add(new ResourceCalendar() { Id = "Calendar2", OwningResourceId = "Resource2", IsVisible = true });
            _calendars.Add(new ResourceCalendar() { Id = "Calendar3", OwningResourceId = "Resource3", IsVisible = true });
            _calendars.Add(new ResourceCalendar() { Id = "Calendar4", OwningResourceId = "Resource4", IsVisible = true });
        }

        private void GenerateAppointments()
        {
            Dispatch(() => _appointments.Clear());

            int id = 0;

            for (int i = 0; i < 5; i++)
            {
                string resource = string.Format("Resource{0}", i);
                string calendar = string.Format("Calendar{0}", i);

                List<Appointment> appointments = new List<Appointment>();

                foreach (AppointmentBO bo in AppointmentBO.Generate(resource, calendar))
                {
                    id++;

                    Appointment appointment = new Appointment()
                    {
                        Id = id.ToString(),
                        OwningResourceId = bo.Resource,
                        OwningCalendarId = bo.Calendar,
                        Subject = i.ToString(),
                        Start = bo.Start.ToUniversalTime(),
                        End = bo.End.ToUniversalTime(),
                        IsLocked = true,
                        Categories = bo.Category,
                        DataItem = bo
                    };

                    appointments.Add(appointment);
                }

                Action action = () =>
                    {
                        foreach (Appointment appointment in appointments)
                            _appointments.Add(appointment);
                    };

                Dispatch(action);
            }
        }

        private void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;

            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }

        private void Dispatch(Action action)
        {
            if (_dispatcher.HasShutdownStarted || _dispatcher.HasShutdownFinished)
                return;

            _dispatcher.Invoke(action);
        }

        public void SetMemoryUsage()
        {
            Memory = Process.GetCurrentProcess().PrivateMemorySize64 / 1048576;
        }
    }
}
